@@ -34,6 +34,11 @@ class DetectorGroup {
       std::vector<std::unique_ptr<BasePlugin>> detectors);
   ~DetectorGroup() = default;
 
+  /*
+   * Prerun all plugins in this detector group.
+   */
+  void prerun(OomdContext& context);
+
   /*
    * @return true if no @class Detector returns PluginRet::STOP.
    */
