@@ -21,16 +21,12 @@
 
 #include "oomd/Log.h"
 #include "oomd/Stats.h"
-#include "oomd/include/Assert.h"
 #include "oomd/include/CoreStats.h"
 
 namespace Oomd {
 namespace Engine {
 
-Engine::Engine(
-    MonitoredResources resources,
-    std::vector<std::unique_ptr<Ruleset>> rulesets)
-    : resources_(std::move(resources)) {
+Engine::Engine(std::vector<std::unique_ptr<Ruleset>> rulesets) {
   for (auto& rs : rulesets) {
     if (rs) {
       rulesets_.emplace_back(BaseRuleset{.ruleset = std::move(rs)});
@@ -95,6 +91,18 @@ void Engine::removeDropInConfig(size_t tag) {
   }
 }
 
+void Engine::prerun(OomdContext& context) {
+  for (const auto& base : rulesets_) {
+    for (const auto& dropin : base.dropins) {
+      if (dropin.ruleset) {
+        dropin.ruleset->prerun(context);
+      }
+    }
+
+    base.ruleset->prerun(context);
+  }
+}
+
 void Engine::runOnce(OomdContext& context) {
   uint32_t nr_dropins_run = 0;
 
@@ -113,9 +121,5 @@ void Engine::runOnce(OomdContext& context) {
   Oomd::incrementStat(CoreStats::kNumDropInFired, nr_dropins_run);
 }
 
-const MonitoredResources& Engine::getMonitoredResources() const {
-  return resources_;
-}
-
 } // namespace Engine
 } // namespace Oomd
