@@ -19,6 +19,7 @@
 
 #include <chrono>
 #include <cstdint>
+#include <iostream>
 #include <vector>
 
 namespace Oomd {
@@ -41,6 +42,12 @@ struct DeviceIOStat {
   int64_t wios{0};
   int64_t dbytes{0};
   int64_t dios{0};
+
+  bool operator==(const DeviceIOStat& rhs) const {
+    return dev_id == rhs.dev_id && rbytes == rhs.rbytes &&
+        wbytes == rhs.wbytes && rios == rhs.rios && wios == rhs.wios &&
+        dbytes == rhs.dbytes && dios == rhs.dios;
+  }
 };
 
 using IOStat = std::vector<DeviceIOStat>;
@@ -57,35 +64,12 @@ struct IOCostCoeffs {
 struct ResourcePressure {
   float sec_10{0};
   float sec_60{0};
-  float sec_600{0};
+  float sec_300{0};
   std::optional<std::chrono::microseconds> total{std::nullopt};
-};
 
-// If you update this class with something that could be valuable to know
-// when debugging, please remember to update OomdContext::dumpOomdContext
-// as well.
-class CgroupContext {
- public:
-  ResourcePressure pressure;
-  ResourcePressure io_pressure;
-  int64_t current_usage{0};
-  int64_t average_usage{0};
-  int64_t memory_low{0};
-  int64_t memory_protection{0};
-  int64_t swap_usage{0};
-  int64_t anon_usage{0};
-  int64_t memory_min{0};
-  int64_t memory_high{0};
-  int64_t memory_high_tmp{0};
-  int64_t memory_max{0};
-  float memory_scale{1};
-  int64_t memory_adj{0};
-  double io_cost_cumulative{0}; // Dot product between io stat and coeffs
-  double io_cost_rate{0}; // change of cumulative divided by interval in seconds
-  int64_t nr_dying_descendants{0};
-
-  int64_t effective_usage() const {
-    return current_usage * memory_scale - memory_protection + memory_adj;
+  bool operator==(const ResourcePressure& rhs) const {
+    return sec_10 == rhs.sec_10 && sec_60 == rhs.sec_60 &&
+        sec_300 == rhs.sec_300 && total == rhs.total;
   }
 };
 
@@ -94,4 +78,24 @@ struct SystemContext {
   uint64_t swapused{0};
 };
 
+enum struct KillPreference {
+  PREFER = 1,
+  NORMAL = 0,
+  AVOID = -1,
+};
+inline std::ostream& operator<<(std::ostream& os, KillPreference kp) {
+  switch (kp) {
+    case KillPreference::PREFER:
+      os << "PREFER";
+      break;
+    case KillPreference::NORMAL:
+      os << "NORMAL";
+      break;
+    case KillPreference::AVOID:
+      os << "AVOID";
+      break;
+  }
+  return os;
+}
+
 } // namespace Oomd
