@@ -28,8 +28,8 @@ template <typename Base = BaseKillPlugin>
 class KillPressure : public Base {
  public:
   int init(
-      Engine::MonitoredResources& resources,
-      const Engine::PluginArgs& args) override;
+      const Engine::PluginArgs& args,
+      const PluginConstructionContext& context) override;
 
   Engine::PluginRet run(OomdContext& ctx) override;
 
@@ -47,9 +47,6 @@ class KillPressure : public Base {
   int post_action_delay_{15};
   bool dry_{false};
   bool debug_{false};
-
- private:
-  static auto constexpr kCgroupFs = "/sys/fs/cgroup";
 };
 
 } // namespace Oomd
