@@ -27,8 +27,8 @@ namespace Oomd {
 class MemoryAbove : public Oomd::Engine::BasePlugin {
  public:
   int init(
-      Engine::MonitoredResources& resources,
-      const Engine::PluginArgs& args) override;
+      const Engine::PluginArgs& args,
+      const PluginConstructionContext& context) override;
 
   Engine::PluginRet run(OomdContext& /* unused */) override;
 
