@@ -26,8 +26,8 @@ namespace Oomd {
 class SwapFree : public Oomd::Engine::BasePlugin {
  public:
   int init(
-      Engine::MonitoredResources& /* unused */,
-      const Engine::PluginArgs& args) override;
+      const Engine::PluginArgs& args,
+      const PluginConstructionContext& /* unused */) override;
 
   Engine::PluginRet run(OomdContext& /* unused */) override;
 
