@@ -25,8 +25,8 @@ template <typename Base = BaseSystemdPlugin>
 class SystemdRestart : public Base {
  public:
   int init(
-      Engine::MonitoredResources& resources,
-      const Engine::PluginArgs& args) override;
+      const Engine::PluginArgs& args,
+      const PluginConstructionContext& /* unused */) override;
 
   Engine::PluginRet run(OomdContext& /* unused */) override;
 
