@@ -19,7 +19,7 @@
 
 #include <dirent.h>
 #include <fcntl.h>
-#include <fnmatch.h>
+#include <glob.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -63,6 +63,40 @@ PsiFormat getPsiFormat(const std::vector<std::string>& lines) {
 
 namespace Oomd {
 
+Fs::Fd Fs::Fd::open(const std::string& path, bool read_only) {
+  int flags = read_only ? O_RDONLY : O_WRONLY;
+  return Fd(::open(path.c_str(), flags));
+}
+
+Fs::Fd
+Fs::Fd::openat(const DirFd& dirfd, const std::string& path, bool read_only) {
+  int flags = read_only ? O_RDONLY : O_WRONLY;
+  return Fd(::openat(dirfd.fd(), path.c_str(), flags));
+}
+
+std::optional<uint64_t> Fs::Fd::inode() const {
+  struct ::stat buf;
+  if (::fstat(fd_, &buf) == 0) {
+    return static_cast<uint64_t>(buf.st_ino);
+  }
+  return std::nullopt;
+}
+
+void Fs::Fd::close() const {
+  if (isValid()) {
+    ::close(fd_);
+  }
+}
+
+Fs::DirFd Fs::DirFd::open(const std::string& path) {
+  return DirFd(::open(path.c_str(), O_RDONLY | O_DIRECTORY));
+}
+
+bool Fs::isCgroupValid(const DirFd& dirfd) {
+  // If cgroup.controllers file exists, the cgroup should still be valid
+  return ::faccessat(dirfd.fd(), kControllersFile, F_OK, 0) == 0;
+}
+
 struct Fs::DirEnts Fs::readDir(const std::string& path, int flags) {
   DIR* d;
   struct Fs::DirEnts de;
@@ -120,81 +154,24 @@ bool Fs::isDir(const std::string& path) {
   return false;
 }
 
-/*
- * Return if a string might have something special for fnmatch.
- *
- * This function is simple and can return false-positives, but not
- * false-negatives -- that is, true means "maybe", and false means false.
- * That's ok, since this is only used for optimisations.
- */
-bool Fs::hasGlob(const std::string& s) {
-  return s.find_first_of("*[?") != std::string::npos;
-}
-
-std::unordered_set<std::string> Fs::resolveWildcardPath(
-    const CgroupPath& cgpath) {
-  std::string path = cgpath.absolutePath();
-  std::unordered_set<std::string> ret;
-  if (path.empty()) {
-    return ret;
+std::vector<std::string> Fs::glob(const std::string& pattern, bool dir_only) {
+  glob_t globbuf;
+  std::vector<std::string> ret;
+  int flags = GLOB_NOSORT | GLOB_BRACE;
+  if (dir_only) {
+    flags |= GLOB_ONLYDIR;
   }
-
-  auto parts = Util::split(path, '/');
-  OCHECK_EXCEPT(!parts.empty(), "No parts in " + path);
-
-  std::deque<std::pair<std::string, size_t>> queue;
-
-  // Add initial path piece to begin search on. Start at root.
-  queue.emplace_back("/", 0);
-
-  // Perform a DFS on the entire search space. Note that we pattern
-  // match at each level of the provided path to eliminate "dead"
-  // branches. The algorithm is still O(N) but in practice this will
-  // prevent us from enumerating every entry in the root filesystem.
-  //
-  // We choose DFS because we predict the FS tree is wider than it
-  // is tall. DFS will use less space than BFS in this case because
-  // it does not need to store every node at each level of the tree.
-  while (!queue.empty()) {
-    const auto front = queue.front(); // copy
-    queue.pop_front();
-
-    // Optimisation: If there's no glob and we're not at the end, it must be
-    // intended to be a single dir. It doesn't matter if it actually *is* in
-    // reality, because if it doesn't exist we'll fail later on.
-    if (front.second < parts.size() - 1 && !Fs::hasGlob(parts[front.second])) {
-      queue.emplace_front(
-          front.first + parts[front.second] + "/", front.second + 1);
-      continue;
-    }
-
-    // We can't continue BFS if we've hit a regular file
-    if (!isDir(front.first)) {
-      continue;
-    }
-
-    auto de = readDir(front.first, DE_FILE | DE_DIR);
-    de.files.reserve(de.files.size() + de.dirs.size());
-    de.files.insert(de.files.end(), de.dirs.begin(), de.dirs.end());
-
-    for (const auto& entry : de.files) {
-      if (::fnmatch(parts[front.second].c_str(), entry.c_str(), 0) == 0) {
-        if (front.second == parts.size() - 1) {
-          // We have reached a leaf, add it to the return set
-          ret.emplace(front.first + entry);
-        } else if (front.second < parts.size() - 1) {
-          // There are still more parts of the provided path to search.
-          //
-          // Note that we add the '/' at the end of the new path. This makes
-          // the recursive case easier, as the recursive case need only
-          // add the next part of the path on. Also note the 'emplace_front'
-          // that makes the deque into a stack (thus the DFS).
-          queue.emplace_front(front.first + entry + "/", front.second + 1);
-        }
+  if (0 == ::glob(pattern.c_str(), flags, nullptr, &globbuf)) {
+    for (size_t i = 0; i < globbuf.gl_pathc; i++) {
+      std::string path(globbuf.gl_pathv[i]);
+      // GLOB_ONLYDIR is a hint and we need to double check
+      if (dir_only && !isDir(path)) {
+        continue;
       }
+      ret.emplace_back(std::move(path));
     }
   }
-
+  ::globfree(&globbuf);
   return ret;
 }
 
@@ -226,40 +203,92 @@ std::vector<std::string> Fs::readFileByLine(const std::string& path) {
   return v;
 }
 
-std::vector<std::string> Fs::readControllers(const std::string& path) {
-  std::vector<std::string> controllers;
-  auto lines = readFileByLine(path + "/" + kControllersFile);
-  if (!lines.size()) {
-    return controllers;
+std::vector<std::string> Fs::readFileByLine(Fd&& fd) {
+  if (!fd.isValid()) {
+    return {};
   }
-
-  controllers = Util::split(lines[0], ' ');
-
-  return controllers;
+  auto fp = fdopen(fd.fd(), "r");
+  if (fp == nullptr) {
+    return {};
+  }
+  std::vector<std::string> v;
+  char* line = nullptr;
+  size_t len = 0;
+  ssize_t read;
+  while ((read = ::getline(&line, &len, fp)) != -1) {
+    OCHECK(line != nullptr);
+    if (read > 0 && line[read - 1] == '\n') {
+      v.emplace_back(line, read - 1);
+    } else {
+      v.emplace_back(line);
+    }
+  }
+  ::free(line);
+  ::fclose(fp);
+  return v;
 }
 
-std::vector<int> Fs::getPids(const std::string& path, bool recursive) {
+std::vector<std::string> Fs::readControllers(const std::string& path) {
+  auto lines = readFileByLine(path + "/" + kControllersFile);
+  if (!lines.size()) {
+    return {};
+  }
+  return Util::split(lines[0], ' ');
+}
+
+std::vector<std::string> Fs::readControllersAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kControllersFile));
+  if (!lines.size()) {
+    return {};
+  }
+  return Util::split(lines[0], ' ');
+}
+
+std::vector<int> Fs::getPids(const std::string& path) {
   std::vector<int> pids;
-  auto de = readDir(path, DE_FILE | DE_DIR);
-  if (std::any_of(de.files.begin(), de.files.end(), [](const std::string& s) {
-        return s == kProcsFile;
-      })) {
-    auto str_pids = readFileByLine(path + "/" + kProcsFile);
-    for (const auto& sp : str_pids) {
-      pids.push_back(std::stoi(sp));
-    }
+  auto str_pids = readFileByLine(path + "/" + kProcsFile);
+  for (const auto& sp : str_pids) {
+    pids.push_back(std::stoi(sp));
   }
-
-  if (recursive) {
-    for (const auto& dir : de.dirs) {
-      auto recursive_pids = getPids(path + "/" + dir, true);
-      pids.insert(pids.end(), recursive_pids.begin(), recursive_pids.end());
-    }
-  }
-
   return pids;
 }
 
+std::vector<int> Fs::getPidsAt(const DirFd& dirfd) {
+  std::vector<int> pids;
+  auto str_pids = readFileByLine(Fs::Fd::openat(dirfd, kProcsFile));
+  for (const auto& sp : str_pids) {
+    pids.push_back(std::stoi(sp));
+  }
+  return pids;
+}
+
+bool Fs::readIsPopulatedFromLines(const std::vector<std::string>& lines) {
+  for (const auto& line : lines) {
+    std::vector<std::string> toks = Util::split(line, ' ');
+    if (toks.size() == 2 && toks[0] == "populated") {
+      if (toks[1] == "1") {
+        return true;
+      } else if (toks[1] == "0") {
+        return false;
+      } else {
+        throw bad_control_file("invalid format");
+      }
+    }
+  }
+
+  throw bad_control_file("invalid format");
+}
+
+bool Fs::readIsPopulated(const std::string& path) {
+  auto lines = readFileByLine(path + "/" + kEventsFile);
+  return readIsPopulatedFromLines(lines);
+}
+
+bool Fs::readIsPopulatedAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kEventsFile));
+  return readIsPopulatedFromLines(lines);
+}
+
 std::string Fs::pressureTypeToString(PressureType type) {
   switch (type) {
     case PressureType::SOME:
@@ -270,11 +299,9 @@ std::string Fs::pressureTypeToString(PressureType type) {
   throw std::runtime_error("Invalid PressureType. Code should not be reached");
 }
 
-ResourcePressure Fs::readRespressure(
-    const std::string& path,
+ResourcePressure Fs::readRespressureFromLines(
+    const std::vector<std::string>& lines,
     PressureType type) {
-  auto lines = readFileByLine(path);
-
   auto type_name = pressureTypeToString(type);
   size_t pressure_line_index = 0;
   switch (type) {
@@ -294,20 +321,15 @@ ResourcePressure Fs::readRespressure(
       // full avg10=0.22 avg60=0.16 avg300=1.08 total=58464525
       std::vector<std::string> toks =
           Util::split(lines[pressure_line_index], ' ');
-      OCHECK_EXCEPT(
-          toks[0] == type_name, bad_control_file(path + ": invalid format"));
+      OCHECK_EXCEPT(toks[0] == type_name, bad_control_file("invalid format"));
       std::vector<std::string> avg10 = Util::split(toks[1], '=');
-      OCHECK_EXCEPT(
-          avg10[0] == "avg10", bad_control_file(path + ": invalid format"));
+      OCHECK_EXCEPT(avg10[0] == "avg10", bad_control_file("invalid format"));
       std::vector<std::string> avg60 = Util::split(toks[2], '=');
-      OCHECK_EXCEPT(
-          avg60[0] == "avg60", bad_control_file(path + ": invalid format"));
+      OCHECK_EXCEPT(avg60[0] == "avg60", bad_control_file("invalid format"));
       std::vector<std::string> avg300 = Util::split(toks[3], '=');
-      OCHECK_EXCEPT(
-          avg300[0] == "avg300", bad_control_file(path + ": invalid format"));
+      OCHECK_EXCEPT(avg300[0] == "avg300", bad_control_file("invalid format"));
       std::vector<std::string> total = Util::split(toks[4], '=');
-      OCHECK_EXCEPT(
-          total[0] == "total", bad_control_file(path + ": invalid format"));
+      OCHECK_EXCEPT(total[0] == "total", bad_control_file("invalid format"));
 
       return ResourcePressure{
           std::stof(avg10[1]),
@@ -324,8 +346,7 @@ ResourcePressure Fs::readRespressure(
       // full 0.00 0.03 0.05
       std::vector<std::string> toks =
           Util::split(lines[pressure_line_index + 1], ' ');
-      OCHECK_EXCEPT(
-          toks[0] == type_name, bad_control_file(path + ": invalid format"));
+      OCHECK_EXCEPT(toks[0] == type_name, bad_control_file("invalid format"));
 
       return ResourcePressure{
           std::stof(toks[1]),
@@ -336,45 +357,56 @@ ResourcePressure Fs::readRespressure(
     }
     case PsiFormat::MISSING:
       // Missing the control file
-      throw bad_control_file(path + ": missing file");
+      throw bad_control_file("missing file");
     case PsiFormat::INVALID:
-      throw bad_control_file(path + ": invalid format");
+      throw bad_control_file("invalid format");
   }
 
   // To silence g++ compiler warning about enums
   throw std::runtime_error("Not all enums handled");
 }
 
+int64_t Fs::readRootMemcurrent() {
+  auto meminfo = getMeminfo("/proc/meminfo");
+  return meminfo["MemTotal"] - meminfo["MemFree"];
+}
+
 int64_t Fs::readMemcurrent(const std::string& path) {
-  if (path == "/") {
-    auto meminfo = getMeminfo("/proc/meminfo");
-    return meminfo["MemTotal"] - meminfo["MemFree"];
-  } else {
-    auto lines = readFileByLine(path + "/" + kMemCurrentFile);
-    OCHECK_EXCEPT(lines.size() == 1, bad_control_file(path + ": missing file"));
-    return static_cast<int64_t>(std::stoll(lines[0]));
+  auto lines = readFileByLine(path + "/" + kMemCurrentFile);
+  OCHECK_EXCEPT(lines.size() == 1, bad_control_file(path + ": missing file"));
+  return static_cast<int64_t>(std::stoll(lines[0]));
+}
+
+int64_t Fs::readMemcurrentAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemCurrentFile));
+  OCHECK_EXCEPT(lines.size() == 1, bad_control_file("missing file"));
+  return static_cast<int64_t>(std::stoll(lines[0]));
+}
+
+ResourcePressure Fs::readRootMempressure(PressureType type) {
+  try {
+    auto lines = readFileByLine("/proc/pressure/memory");
+    return readRespressureFromLines(lines, type);
+  } catch (const bad_control_file& e) {
+    auto lines = readFileByLine("/proc/mempressure");
+    return readRespressureFromLines(lines, type);
   }
 }
 
 ResourcePressure Fs::readMempressure(
     const std::string& path,
     PressureType type) {
-  if (path == "/") {
-    try {
-      return readRespressure("/proc/pressure/memory", type);
-    } catch (const bad_control_file& e) {
-      return readRespressure("/proc/mempressure", type);
-    }
-  } else {
-    return readRespressure(path + "/" + kMemPressureFile, type);
-  }
+  auto lines = readFileByLine(path + "/" + kMemPressureFile);
+  return readRespressureFromLines(lines, type);
 }
 
-int64_t Fs::readMinMaxLowHigh(
-    const std::string& path,
-    const std::string& file) {
-  auto lines = readFileByLine(path + "/" + file);
-  OCHECK_EXCEPT(lines.size() == 1, bad_control_file(path + ": missing file"));
+ResourcePressure Fs::readMempressureAt(const DirFd& dirfd, PressureType type) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemPressureFile));
+  return readRespressureFromLines(lines, type);
+}
+
+int64_t Fs::readMinMaxLowHighFromLines(const std::vector<std::string>& lines) {
+  OCHECK_EXCEPT(lines.size() == 1, bad_control_file("missing file"));
   if (lines[0] == "max") {
     return std::numeric_limits<int64_t>::max();
   }
@@ -382,31 +414,63 @@ int64_t Fs::readMinMaxLowHigh(
 }
 
 int64_t Fs::readMemlow(const std::string& path) {
-  return Fs::readMinMaxLowHigh(path, kMemLowFile);
+  auto lines = readFileByLine(path + "/" + kMemLowFile);
+  return Fs::readMinMaxLowHighFromLines(lines);
+}
+
+int64_t Fs::readMemlowAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemLowFile));
+  return Fs::readMinMaxLowHighFromLines(lines);
 }
 
 int64_t Fs::readMemhigh(const std::string& path) {
-  return Fs::readMinMaxLowHigh(path, kMemHighFile);
+  auto lines = readFileByLine(path + "/" + kMemHighFile);
+  return Fs::readMinMaxLowHighFromLines(lines);
+}
+
+int64_t Fs::readMemhighAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemHighFile));
+  return Fs::readMinMaxLowHighFromLines(lines);
 }
 
 int64_t Fs::readMemmax(const std::string& path) {
-  return Fs::readMinMaxLowHigh(path, kMemMaxFile);
+  auto lines = readFileByLine(path + "/" + kMemMaxFile);
+  return Fs::readMinMaxLowHighFromLines(lines);
 }
 
-int64_t Fs::readMemhightmp(const std::string& path) {
-  auto lines = readFileByLine(path + "/" + kMemHighTmpFile);
-  OCHECK_EXCEPT(lines.size() == 1, bad_control_file(path + ": missing file"));
+int64_t Fs::readMemmaxAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemMaxFile));
+  return Fs::readMinMaxLowHighFromLines(lines);
+}
+
+int64_t Fs::readMemhightmpFromLines(const std::vector<std::string>& lines) {
+  OCHECK_EXCEPT(lines.size() == 1, bad_control_file("missing file"));
   auto tokens = Util::split(lines[0], ' ');
-  OCHECK_EXCEPT(
-      tokens.size() == 2, bad_control_file(path + ": invalid format"));
+  OCHECK_EXCEPT(tokens.size() == 2, bad_control_file("invalid format"));
   if (tokens[0] == "max") {
     return std::numeric_limits<int64_t>::max();
   }
   return static_cast<int64_t>(std::stoll(tokens[0]));
 }
 
+int64_t Fs::readMemhightmp(const std::string& path) {
+  auto lines = readFileByLine(path + "/" + kMemHighTmpFile);
+  return readMemhightmpFromLines(lines);
+}
+
+int64_t Fs::readMemhightmpAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemHighTmpFile));
+  return readMemhightmpFromLines(lines);
+}
+
 int64_t Fs::readMemmin(const std::string& path) {
-  return Fs::readMinMaxLowHigh(path, kMemMinFile);
+  auto lines = readFileByLine(path + "/" + kMemMinFile);
+  return Fs::readMinMaxLowHighFromLines(lines);
+}
+
+int64_t Fs::readMemminAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemMinFile));
+  return Fs::readMinMaxLowHighFromLines(lines);
 }
 
 int64_t Fs::readSwapCurrent(const std::string& path) {
@@ -420,6 +484,12 @@ int64_t Fs::readSwapCurrent(const std::string& path) {
   }
 }
 
+int64_t Fs::readSwapCurrentAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemSwapCurrentFile));
+  // The swap controller can be disabled via CONFIG_MEMCG_SWAP=n
+  return lines.size() == 1 ? static_cast<int64_t>(std::stoll(lines[0])) : 0;
+}
+
 std::unordered_map<std::string, int64_t> Fs::getVmstat(
     const std::string& path) {
   auto lines = readFileByLine(path);
@@ -460,13 +530,12 @@ std::unordered_map<std::string, int64_t> Fs::getMeminfo(
   return map;
 }
 
-std::unordered_map<std::string, int64_t> Fs::getMemstatLike(
-    const std::string& file) {
+std::unordered_map<std::string, int64_t> Fs::getMemstatLikeFromLines(
+    const std::vector<std::string>& lines) {
   char name[256] = {0};
   uint64_t val;
   std::unordered_map<std::string, int64_t> map;
 
-  auto lines = readFileByLine(file);
   for (const auto& line : lines) {
     int ret = sscanf(line.c_str(), "%255s %" SCNu64 "\n", name, &val);
     if (ret == 2) {
@@ -479,22 +548,33 @@ std::unordered_map<std::string, int64_t> Fs::getMemstatLike(
 
 std::unordered_map<std::string, int64_t> Fs::getMemstat(
     const std::string& path) {
-  return getMemstatLike(path + "/" + kMemStatFile);
+  auto lines = readFileByLine(path + "/" + kMemStatFile);
+  return getMemstatLikeFromLines(lines);
+}
+
+std::unordered_map<std::string, int64_t> Fs::getMemstatAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemStatFile));
+  return getMemstatLikeFromLines(lines);
+}
+
+ResourcePressure Fs::readRootIopressure(PressureType type) {
+  auto lines = readFileByLine("/proc/pressure/io");
+  return readRespressureFromLines(lines, type);
 }
 
 ResourcePressure Fs::readIopressure(
     const std::string& path,
     PressureType type) {
-  if (path == "/") {
-    return readRespressure("/proc/pressure/io", type);
-  } else {
-    return readRespressure(path + "/" + kIoPressureFile, type);
-  }
+  auto lines = readFileByLine(path + "/" + kIoPressureFile);
+  return readRespressureFromLines(lines, type);
 }
 
-IOStat Fs::readIostat(const std::string& path) {
-  const auto& io_stat_path = path + "/" + kIoStatFile;
-  auto lines = readFileByLine(io_stat_path);
+ResourcePressure Fs::readIopressureAt(const DirFd& dirfd, PressureType type) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kIoPressureFile));
+  return readRespressureFromLines(lines, type);
+}
+
+IOStat Fs::readIostatFromLines(const std::vector<std::string>& lines) {
   std::vector<DeviceIOStat> io_stat;
   io_stat.reserve(lines.size());
 
@@ -517,58 +597,88 @@ IOStat Fs::readIostat(const std::string& path) {
         &dev_io_stat.dbytes,
         &dev_io_stat.dios);
 
-    OCHECK_EXCEPT(ret == 8, bad_control_file(path + ": invalid format"));
+    OCHECK_EXCEPT(ret == 8, bad_control_file("invalid format"));
     dev_io_stat.dev_id = std::to_string(major) + ":" + std::to_string(minor);
     io_stat.push_back(dev_io_stat);
   }
   return io_stat;
 }
 
-void Fs::writeMemhigh(const std::string& path, int64_t value) {
+IOStat Fs::readIostat(const std::string& path) {
+  auto lines = readFileByLine(path + "/" + kIoStatFile);
+  return readIostatFromLines(lines);
+}
+
+IOStat Fs::readIostatAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kIoStatFile));
+  return readIostatFromLines(lines);
+}
+
+void Fs::writeControlFileAt(Fd&& fd, const std::string& content) {
   char buf[1024];
   buf[0] = '\0';
-  auto file_name = path + "/" + kMemHighFile;
-  auto fd = ::open(file_name.c_str(), O_WRONLY);
-  if (fd < 0) {
+  if (!fd.isValid()) {
     throw bad_control_file(
-        file_name + ": open failed: " + ::strerror_r(errno, buf, sizeof(buf)));
+        std::string{"open failed: "} + ::strerror_r(errno, buf, sizeof(buf)));
   }
-  auto val_str = std::to_string(value);
-  auto ret = Util::writeFull(fd, val_str.c_str(), val_str.size());
-  ::close(fd);
+  auto ret = Util::writeFull(fd.fd(), content.c_str(), content.size());
   if (ret < 0) {
     throw bad_control_file(
-        file_name + ": write failed: " + ::strerror_r(errno, buf, sizeof(buf)));
+        std::string{"write failed: "} + ::strerror_r(errno, buf, sizeof(buf)));
   }
 }
 
+void Fs::writeMemhigh(const std::string& path, int64_t value) {
+  auto val_str = std::to_string(value);
+  writeControlFileAt(Fs::Fd::open(path + "/" + kMemHighFile, false), val_str);
+}
+
+void Fs::writeMemhighAt(const DirFd& dirfd, int64_t value) {
+  auto val_str = std::to_string(value);
+  writeControlFileAt(Fs::Fd::openat(dirfd, kMemHighFile, false), val_str);
+}
+
 void Fs::writeMemhightmp(
     const std::string& path,
     int64_t value,
     std::chrono::microseconds duration) {
-  char buf[1024];
-  buf[0] = '\0';
-  auto file_name = path + "/" + kMemHighTmpFile;
-  auto fd = ::open(file_name.c_str(), O_WRONLY);
-  if (fd < 0) {
-    throw bad_control_file(
-        file_name + ": open failed: " + ::strerror_r(errno, buf, sizeof(buf)));
-  }
   auto val_str = std::to_string(value) + " " + std::to_string(duration.count());
-  auto ret = Util::writeFull(fd, val_str.c_str(), val_str.size());
-  ::close(fd);
-  if (ret < 0) {
-    throw bad_control_file(
-        file_name + ": write failed: " + ::strerror_r(errno, buf, sizeof(buf)));
-  }
+  writeControlFileAt(
+      Fs::Fd::open(path + "/" + kMemHighTmpFile, false), val_str);
+}
+
+void Fs::writeMemhightmpAt(
+    const DirFd& dirfd,
+    int64_t value,
+    std::chrono::microseconds duration) {
+  auto val_str = std::to_string(value) + " " + std::to_string(duration.count());
+  writeControlFileAt(Fs::Fd::openat(dirfd, kMemHighTmpFile, false), val_str);
 }
 
 int64_t Fs::getNrDyingDescendants(const std::string& path) {
-  auto map = getMemstatLike(path + "/" + kCgroupStatFile);
+  auto lines = readFileByLine(path + "/" + kCgroupStatFile);
+  auto map = getMemstatLikeFromLines(lines);
   // Will return 0 for missing entries
   return map["nr_dying_descendants"];
 }
 
+int64_t Fs::getNrDyingDescendantsAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kCgroupStatFile));
+  auto map = getMemstatLikeFromLines(lines);
+  // Will return 0 for missing entries
+  return map["nr_dying_descendants"];
+}
+
+KillPreference Fs::readKillPreferenceAt(const DirFd& path) {
+  if (Fs::hasxattrAt(path, kOomdPreferXAttr)) {
+    return KillPreference::PREFER;
+  } else if (Fs::hasxattrAt(path, kOomdAvoidXAttr)) {
+    return KillPreference::AVOID;
+  } else {
+    return KillPreference::NORMAL;
+  }
+}
+
 bool Fs::setxattr(
     const std::string& path,
     const std::string& attr,
@@ -593,6 +703,14 @@ std::string Fs::getxattr(const std::string& path, const std::string& attr) {
   return val;
 }
 
+bool Fs::hasxattr(const std::string& path, const std::string& attr) {
+  return ::getxattr(path.c_str(), attr.c_str(), nullptr, 0) >= 0;
+}
+
+bool Fs::hasxattrAt(const DirFd& dirfd, const std::string& attr) {
+  return ::fgetxattr(dirfd.fd(), attr.c_str(), nullptr, 0) >= 0;
+}
+
 bool Fs::isUnderParentPath(
     const std::string& parent_path,
     const std::string& path) {
