@@ -18,12 +18,12 @@
 #pragma once
 
 #include <exception>
+#include <stdexcept>
 #include <string>
 #include <unordered_map>
 #include <unordered_set>
 #include <vector>
 
-#include "oomd/include/CgroupPath.h"
 #include "oomd/include/Types.h"
 
 namespace Oomd {
@@ -40,6 +40,7 @@ class Fs {
   static constexpr auto kControllersFile = "cgroup.controllers";
   static constexpr auto kSubtreeControlFile = "cgroup.subtree_control";
   static constexpr auto kProcsFile = "cgroup.procs";
+  static constexpr auto kEventsFile = "cgroup.events";
   static constexpr auto kMemCurrentFile = "memory.current";
   static constexpr auto kMemPressureFile = "memory.pressure";
   static constexpr auto kMemLowFile = "memory.low";
@@ -54,6 +55,8 @@ class Fs {
   static constexpr auto kIoStatFile = "io.stat";
   static constexpr auto kDeviceTypeDir = "queue";
   static constexpr auto kDeviceTypeFile = "rotational";
+  static constexpr auto kOomdPreferXAttr = "trusted.oomd_prefer";
+  static constexpr auto kOomdAvoidXAttr = "trusted.oomd_avoid";
 
   struct DirEnts {
     std::vector<std::string> dirs;
@@ -70,6 +73,60 @@ class Fs {
     FULL,
   };
 
+  class DirFd;
+  /*
+   * Wrapper class for file descriptor that supports auto closing.
+   * Fd is opened read-only as currently that's the only use case.
+   */
+  class Fd {
+   public:
+    static Fd open(const std::string& path, bool read_only = true);
+    static Fd
+    openat(const DirFd& dirfd, const std::string& path, bool read_only = true);
+
+    Fd() = default;
+    Fd(const Fd& other) = delete;
+    Fd(Fd&& other) noexcept {
+      *this = std::move(other);
+    }
+    Fd& operator=(const Fd& other) = delete;
+    Fd& operator=(Fd&& other) {
+      fd_ = other.fd_;
+      other.fd_ = -1;
+      return *this;
+    }
+    ~Fd() {
+      this->close();
+    }
+
+    int fd() const {
+      return fd_;
+    }
+    bool isValid() const {
+      return fd_ >= 0;
+    }
+    // Return inode of the fd, or nullopt if anything fails
+    std::optional<uint64_t> inode() const;
+
+   protected:
+    explicit Fd(int fd) : fd_(fd) {}
+    void close() const;
+    int fd_{-1};
+  };
+
+  /*
+   * Wrapper class for directory file descriptor that supports auto closing.
+   */
+  class DirFd : public Fd {
+   public:
+    static DirFd open(const std::string& path);
+
+   protected:
+    explicit DirFd(int fd) : Fd(fd) {}
+  };
+
+  static bool isCgroupValid(const DirFd& dirfd);
+
   /*
    * Reads a directory and returns the names of the requested entry types
    * Won't return any dotfiles (including ./ and ../)
@@ -85,8 +142,9 @@ class Fs {
    * Takes a fully qualified and wildcarded path and returns a set of
    * resolved fully qualified paths.
    */
-  static std::unordered_set<std::string> resolveWildcardPath(
-      const CgroupPath& path);
+  static std::vector<std::string> glob(
+      const std::string& pattern,
+      bool dir_only = false);
 
   /*
    * Path aware prefix removal.
@@ -97,44 +155,81 @@ class Fs {
 
   /* Reads a file and returns a newline separated vector of strings */
   static std::vector<std::string> readFileByLine(const std::string& path);
+  /*
+   * Same as variant taking string as argument except rvalue Fd is used, which
+   * will be closed right after the call as it is read as a whole and we don't
+   * seek offsets on Fd.
+   */
+  static std::vector<std::string> readFileByLine(Fd&& fd);
 
   static std::vector<std::string> readControllers(const std::string& path);
+  static std::vector<std::string> readControllersAt(const DirFd& dirfd);
 
-  static std::vector<int> getPids(
-      const std::string& path,
-      bool recursive = false);
+  static std::vector<int> getPids(const std::string& path);
+  static std::vector<int> getPidsAt(const DirFd& dirfd);
+
+  static bool readIsPopulatedFromLines(const std::vector<std::string>& lines);
+  static bool readIsPopulated(const std::string& path);
+  static bool readIsPopulatedAt(const DirFd& dirfd);
 
   static std::string pressureTypeToString(PressureType type);
   /* Helpers to read PSI files */
-  static ResourcePressure readRespressure(
-      const std::string& path,
+  static ResourcePressure readRespressureFromLines(
+      const std::vector<std::string>& lines,
       PressureType type = PressureType::FULL);
+  static int64_t readRootMemcurrent();
   static int64_t readMemcurrent(const std::string& path);
+  static int64_t readMemcurrentAt(const DirFd& dirfd);
+  static ResourcePressure readRootMempressure(
+      PressureType type = PressureType::FULL);
   static ResourcePressure readMempressure(
       const std::string& path,
       PressureType type = PressureType::FULL);
-  static int64_t readMinMaxLowHigh(
-      const std::string& path,
-      const std::string& file);
+  static ResourcePressure readMempressureAt(
+      const DirFd& dirfd,
+      PressureType type = PressureType::FULL);
+  static int64_t readMinMaxLowHighFromLines(
+      const std::vector<std::string>& lines);
+  static int64_t readMemhightmpFromLines(const std::vector<std::string>& lines);
   static int64_t readMemlow(const std::string& path);
+  static int64_t readMemlowAt(const DirFd& dirfd);
   static int64_t readMemhigh(const std::string& path);
+  static int64_t readMemhighAt(const DirFd& dirfd);
   static int64_t readMemmax(const std::string& path);
+  static int64_t readMemmaxAt(const DirFd& dirfd);
   static int64_t readMemhightmp(const std::string& path);
+  static int64_t readMemhightmpAt(const DirFd& dirfd);
   static int64_t readMemmin(const std::string& path);
+  static int64_t readMemminAt(const DirFd& dirfd);
   static int64_t readSwapCurrent(const std::string& path);
+  static int64_t readSwapCurrentAt(const DirFd& dirfd);
+  static ResourcePressure readRootIopressure(
+      PressureType type = PressureType::FULL);
   static ResourcePressure readIopressure(
       const std::string& path,
       PressureType type = PressureType::FULL);
+  static ResourcePressure readIopressureAt(
+      const DirFd& dirfd,
+      PressureType type = PressureType::FULL);
 
   static void writeMemhigh(const std::string& path, int64_t value);
+  static void writeMemhighAt(const DirFd& dirfd, int64_t value);
   static void writeMemhightmp(
       const std::string& path,
       int64_t value,
       std::chrono::microseconds duration);
+  static void writeMemhightmpAt(
+      const DirFd& dirfd,
+      int64_t value,
+      std::chrono::microseconds duration);
 
   static int64_t getNrDyingDescendants(const std::string& path);
+  static int64_t getNrDyingDescendantsAt(const DirFd& dirfd);
+
+  static KillPreference readKillPreferenceAt(const DirFd& path);
 
   static IOStat readIostat(const std::string& path);
+  static IOStat readIostatAt(const DirFd& dirfd);
 
   static std::unordered_map<std::string, int64_t> getVmstat(
       const std::string& path = "/proc/vmstat");
@@ -144,6 +239,8 @@ class Fs {
 
   static std::unordered_map<std::string, int64_t> getMemstat(
       const std::string& path);
+  static std::unordered_map<std::string, int64_t> getMemstatAt(
+      const DirFd& dirfd);
 
   // Return root part of cgroup2 from /proc/mounts/
   static std::string getCgroup2MountPoint(
@@ -161,16 +258,19 @@ class Fs {
       const std::string& val);
   static std::string getxattr(const std::string& path, const std::string& attr);
 
+  static bool hasxattr(const std::string& path, const std::string& attr);
+  static bool hasxattrAt(const DirFd& dirfd, const std::string& attr);
+
   // Return if device is SSD or HDD given its id in <major>:<minor> format
   static DeviceType getDeviceType(
       const std::string& dev_id,
       const std::string& path = "/sys/dev/block");
 
-  static bool hasGlob(const std::string& s);
-
  private:
-  static std::unordered_map<std::string, int64_t> getMemstatLike(
-      const std::string& file);
+  static std::unordered_map<std::string, int64_t> getMemstatLikeFromLines(
+      const std::vector<std::string>& lines);
+  static IOStat readIostatFromLines(const std::vector<std::string>& lines);
+  static void writeControlFileAt(Fd&& fd, const std::string& content);
 };
 
 } // namespace Oomd
